#!/usr/bin/env python
# -*- coding: utf-8 -*-

''' https://www.gimpscripts.net/2020/10/text-for-design-ver-01.html
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

Attention: Do not use these characters \/?:*.><| to save the as file
'''
# Rel 0.1: 22/05/2023 Initial release. This plugin is part of the Font-to-layers.py


import gimp
from gimpfu import *
import sys, os, gettext
import math

whoiam = os.path.abspath(sys.argv[0])
# initialize internationalisation in a user-locale (by rob_brz1)
locale_directory = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "Unicode_to_shape_Animals", locale_directory, unicode=True )
	
def font_to_comic_balloon(img,layer,create_new, fill, text_color, text_pattern, font_b, chars, size, transform, angle, visible, visible_l):

	
    if font_b==0:
        font="TalkBalloon"
    if font_b==1:
       font="TENNESSEE Thin"		
################################################################################

    if create_new==1:		
        img = pdb.gimp_image_new(size, size, 0)
        display = pdb.gimp_display_new(img)
        drawable = pdb.gimp_layer_new(img, size, size, 1, "name", 100, 0)
        pdb.gimp_image_insert_layer(img, drawable, None, 0)
		
################################################################################
		
    print font, chars, size
    chars = chars.decode("utf-8")
    parts = font.split(" ")
    if parts[-1].isdigit():
        fontparts  = parts[:-1]
        font = string.join(fontparts, " ")


    max_width = 0.0
    max_height = 0.0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        
        if width > max_width:
            max_width = width
        if height > max_height:
            max_height = height


    layer_count = 0
    drawables = []
    new_max_height = 0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        #Tin Tran: here's a bug with WideAwakeBlack.ttf reported by Wallace.
        #So here we're calling for max_width and max_height instead of width and height then we'll crop it and resize it later.
		#The wideawakeblack.ttf is just a mess with width not returning properly.
        drawable = gimp.Layer(img, "%s" % char, max_width, max_height, RGBA_IMAGE, 100, NORMAL_MODE)
        img.add_layer(drawable, layer_count)
        layer_count = layer_count + 1
        #pdb.gimp_edit_clear(drawable)

        font_layer = pdb.gimp_text_fontname(img, drawable,  0.0, 0.0, char, 0, 1, size, 1, font)
        pdb.gimp_layer_resize_to_image_size(drawable)
        if fill==0:
             pdb.gimp_text_layer_set_color(font_layer, text_color)
        #gimp.set_foreground(text_color)
	
# vectors	
        new_vectors = pdb.gimp_vectors_new(img, font_layer)
		
        if transform <=7:		
            x=pdb.gimp_vectors_new_from_text_layer(img, font_layer)
            pdb.gimp_image_insert_vectors(img, x, None, -1)
            pdb.gimp_image_lower_vectors_to_bottom(img, x)
            active_vectors = pdb.gimp_image_get_active_vectors(img)
			
        #active_vectors = pdb.gimp_image_get_active_vectors(img)	
        w = pdb.gimp_drawable_width(font_layer)
        h = pdb.gimp_drawable_height(font_layer)		
        if transform==1:
            pdb.gimp_item_transform_flip_simple(font_layer, 0, FALSE, w/2)
            pdb.gimp_item_transform_flip_simple(x, 0, FALSE,w/2)
			
        if transform==2:
            pdb.gimp_item_transform_flip_simple(font_layer, 1, FALSE, h/2)
            pdb.gimp_item_transform_flip_simple(active_vectors, 1, FALSE, h/2)

        if transform==3:
            pdb.gimp_item_transform_rotate_simple(font_layer, 0, FALSE, w/2, h/2)
            pdb.gimp_item_transform_rotate_simple(active_vectors, 0, FALSE, w/2, h/2)
			
        if transform==4:
            pdb.gimp_item_transform_rotate_simple(font_layer, 1, FALSE, w/2, h/2)
            pdb.gimp_item_transform_rotate_simple(active_vectors, 1, FALSE, w/2, h/2)
			
        if transform==5:
            pdb.gimp_item_transform_rotate_simple(font_layer, 2, FALSE, w/2, h/2)
            pdb.gimp_item_transform_rotate_simple(active_vectors, 2, FALSE, w/2, h/2)
			
        if transform==6:		
            pdb.gimp_item_transform_rotate(font_layer, angle*0.01745329252 ,FALSE, w/2, h/2)
            pdb.gimp_item_transform_rotate(active_vectors, angle*0.01745329252, FALSE, w/2, h/2)

			
        if transform==7:

            w = max_width
            #h = max_height	
            h = size
            corner = math.fabs(angle)			
            rad =  corner*0.01745329252
            an = h*math.tan(rad)

            if angle < 0:			
                 pdb.gimp_item_transform_perspective(font_layer, 0, 0,   w, 0,   an, h,    w+an, h)
            else:
                 pdb.gimp_item_transform_perspective(font_layer, an, 0,   w+an, 0,   0, h,    w, h)
            #pdb.gimp_context_set_transform_resize(0)  #TRANSFORM-RESIZE-ADJUST (0), TRANSFORM-RESIZE-CLIP (1), TRANSFORM-RESIZE-CROP (2), TRANSFORM-RESIZE-CROP-WITH-ASPECT (3) }			
            pdb.gimp_image_resize_to_layers(img)	
            pdb.gimp_layer_resize_to_image_size(font_layer)
	
            pdb.gimp_image_select_item(img, 2, font_layer)
            pdb.plug_in_sel2path(img, font_layer)
			
            x = pdb.gimp_image_get_active_vectors(img)
            name = char
            pdb.gimp_item_set_name(x, name)
            pdb.gimp_image_lower_vectors_to_bottom(img, x)
			
				
			


        if fill==1:
            pdb.gimp_image_select_item(img,0,font_layer)
            pdb.gimp_context_set_pattern(text_pattern)
            pdb.gimp_edit_fill(font_layer,4) 
            #pdb.gimp_selection_none(img)
			
        if visible:			
            x.visible = True
			
		
        pdb.gimp_floating_sel_anchor(font_layer)
		#Tin Tran: the bug is with the width, so we'll use autocrop to make it crop to width and restore original height
        pdb.gimp_selection_none(img)		
        pdb.plug_in_autocrop_layer(img,drawable)
        pdb.gimp_selection_none(img)		
        offsetx,offsety = pdb.gimp_drawable_offsets(drawable)
        drawables.append(drawable)
        new_height = offsety + drawable.height
        if new_height > new_max_height:
            new_max_height = new_height
    for drawable in drawables:
        offsetx,offsety = pdb.gimp_drawable_offsets(drawable)
        pdb.gimp_layer_resize(drawable,drawable.width,new_max_height,0,offsety)
        pdb.gimp_image_resize_to_layers(img)		
		
    name = "%s%s"  % ("", "")
    filename = "%s%s" % ("", "")
	
    if visible_l:			
        drawable.visible = True		
	
    print img, drawable, filename, filename, 50, name, max_width, new_max_height, 1, 1, 1, [len(chars)], 1
    #disp = gimp.Display(img)

    return
	
register(
    "font_to_comic_balloon",
    "This plugin creates a transparent (contur) comics balloon\n(on a new layer and path) from dingbat fonts.\nYou can use the Fuzzy Select Tool + Bucket Fill Tool for filling.",
    "Font to comic balloon",
    "MareroQ",
    "GPL",
    "2023",
    "Font to comic balloon...",
    "*",
    [
    (PF_IMAGE, "image", "Input image", None), #current image
    (PF_DRAWABLE, "layer", "Input layer", None), #current layer
    (PF_BOOL,  "create_new", _("Create new image:"), False),
    (PF_OPTION, "fill", _("Filling the contur"), 0,("Color","Patterns")),
    (PF_COLOR, "text_color", "Color:", (0, 0, 0)),	
    (PF_PATTERN, "text_pattern", "Pattern:", ""),
    (PF_OPTION, "font_b", _("Font"), 0,("TalkBalloon","TENNESSEE Thin")),	
    #(PF_FONT, "font", "Font: ", "Times New Roman"),
    (PF_STRING, "text", "Characters: ", "a"),
    (PF_FLOAT, "size", "Size of Font: ", 100.0),
    (PF_OPTION,"flip", _("Transform:"), 0, [ _("DON'T USE"), _("HORIZONTAL "u'\u21e8'""), _("VERTICAL "u'\u21e9'""), _("ROTATE 90"), _("ROTATE 180"), _("ROTATE 270"), _("ROTATE CUSTOM ANGLE (deg.)  "u'\u21bb'""), _("SLOPE CUSTOM ANGLE (deg.) "u'\u21bb'"")]),		
    (PF_SPINNER, "angle", _("Custom angle: "), 0, (-45, 45, 1)),
    (PF_BOOL,  "visible",      "Paths visible:", False),
    (PF_BOOL,  "visible_l",      "Layers visible:", True),		
    ],
    [],
    font_to_comic_balloon, menu="<Image>/Filters/Decor/Comics/",
    domain = ("font_to_comic_balloon", locale_directory))

main()